{% extends "base.html" %}

{% block content %}

<h2>Please fix the auth-groupex request.</h2>

<form method="GET">
    {% csrf_token %}
    <table>
        {{ form.as_table }}
        <tr>
            <td colspan="2">
                <input type="submit" value="Submit" />
            </td>
        </tr>
    </table>
</form>

{% endblock content %}
