{% extends "base.html" %}
{% load url from future %}

{% block content %}
{% if user.is_authenticated %}
    <h3>Welcome, {{ user.get_full_name }}</h3>
    <p><a href="{% url 'logout' %}">Log out </a></p>
{% else %}
    <h3>Authenticate</h3>
        <p><a href="{% url 'authgroupex:login' %}?next=/" />Connect through X.org</a></p>
{% endif %}
{% endblock %}
