{% load staticfiles %}

<html>
  <head>
    <title>django-authgroupex :: Demo</title>
  </head>

  <body>
    {% if messages %}
      <div id="messages">
        <ul class="messages">
          {% for message in messages %}
          <li{% if message.tags %} class="{{ message.tags }}"{% endif %}>{{ message }}</li>
          {% endfor %}
        </ul>
      </div>
    {% endif %}

    {% block content %}
    {% endblock content %}
  </body>
</html>
